import { APP_INITIALIZER, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { RouterModule } from '@angular/router';
import { filter } from 'rxjs/operators';
import { AppComponent } from './app.component';
import { HomeComponent } from './home/home.component';
import { UnauthorizedComponent } from './unauthorized/unauthorized.component';
import { AuthModule, EventTypes, LogLevel, OidcConfigService, PublicEventsService } from 'angular-auth-oidc-client';
export function configureAuth(oidcConfigService: OidcConfigService) {
  return () =>
      oidcConfigService.withConfig({
          stsServer: 'https://log-in.test.miljoeportal.dk/runtime/oauth2',
          redirectUrl: window.location.origin,
          postLogoutRedirectUri: window.location.origin +'/unauthorized/',
          clientId: 'oauth-sampleapp-localhost',
          scope: 'openid offline_access http://www.miljoeportal.dk/roles',
          responseType: 'code',
          silentRenew: true,
          useRefreshToken: true,
          logLevel: LogLevel.Debug,
      });
}

@NgModule({
  declarations: [AppComponent, HomeComponent, UnauthorizedComponent],
  imports: [
      BrowserModule,
      RouterModule.forRoot([
          { path: '', redirectTo: 'home', pathMatch: 'full' },
          { path: 'home', component: HomeComponent },
          { path: 'forbidden', component: UnauthorizedComponent },
          { path: 'unauthorized', component: UnauthorizedComponent },
      ]),
      AuthModule.forRoot(),
  ],
  providers: [
      OidcConfigService,
      {
          provide: APP_INITIALIZER,
          useFactory: configureAuth,
          deps: [OidcConfigService],
          multi: true,
      },
  ],
  bootstrap: [AppComponent],
})
export class AppModule {
  constructor(private readonly eventService: PublicEventsService) {
      this.eventService
          .registerForEvents()
          .pipe(filter((notification) => notification.type === EventTypes.ConfigLoaded))
          .subscribe((config) => {
              console.log('ConfigLoaded', config);
          });
  }
}